from .models import Dashboard
from .session import APISession


class GrafanaError(Exception):
    pass


class Grafana:
    def __init__(self, url, api_key):
        self.session = APISession(url, api_key)

    def __str__(self):
        return self.session.url

    def __repr__(self):
        return f'Grafana("{self.session.url}")'

    def health(self):
        r = self.session.get('/api/health')
        if r.status_code != 200:
            raise GrafanaError('no 200 on /api/health')
        elif r.json()['database'] != 'ok':
            raise GrafanaError('database nok')
        else:
            return True

    def dashboards(self):
        result = []
        # https://grafana.com/docs/grafana/latest/http_api/folder_dashboard_search/
        r = self.session.get('/api/search?query=&starred=false')
        r.raise_for_status()
        search_results = r.json()
        uids = [sr['uid'] for sr in search_results if sr['type'] == 'dash-db']
        for uid in uids:
            d = self.session.get(f'/api/dashboards/uid/{uid}').json()['dashboard']
            result.append(Dashboard(d))
        return result



import logging
import os

import click

from hukudo.logging import LOG_LEVELS, configure_structlog_dev
from .grafana.cli import grafana


@click.group()
@click.option(
    '-l',
    '--log-level',
    type=click.Choice(LOG_LEVELS, case_sensitive=False),
    default=os.environ.get('LOGLEVEL', 'WARNING'),
)
def root(log_level):
    """
    For completion, add this to ~/.bashrc:

        eval "$(_HUKUDO_COMPLETE=bash_source hukudo)"

    See also https://click.palletsprojects.com/en/8.1.x/shell-completion/
    """
    configure_structlog_dev(log_level)
    logging.basicConfig(level=log_level, format='%(msg)s')


# noinspection PyTypeChecker
root.add_command(grafana)


def main():
    root()



import os

import click
import structlog

from hukudo.grafana import Grafana

logger = structlog.get_logger()


@click.group()
@click.option('-u', '--url', required=True, envvar='GRAFANA_URL')
@click.option('-k', '--api-key', required=True, envvar='GRAFANA_API_KEY')
@click.pass_context
def grafana(ctx, url, api_key):
    ctx.obj = Grafana(
        url=url,
        api_key=api_key,
    )
    log = logger.bind(instance=ctx.obj)

    try:
        root_ca = os.environ['GRAFANA_CLIENT_ROOT_CA']
        log.debug('CA', path=root_ca)
        ctx.obj.session.verify = root_ca
    except KeyError:
        pass

    try:
        crt = os.environ['GRAFANA_CLIENT_CRT']
        key = os.environ['GRAFANA_CLIENT_KEY']
        log.debug('client cert', crt=crt, key=key)
        ctx.obj.session.cert = (crt, key)
    except KeyError:
        pass


@grafana.group()
def dashboards():
    pass


@dashboards.command()
@click.pass_context
def export(ctx):
    """
    Exports dashboards as json files to a directory named after the Grafana domain.
    """
    log = logger.bind(instance=ctx.obj)
    log.info('export')
    from pathlib import Path
    from urllib.parse import urlparse

    root = Path('')
    grafana: Grafana = ctx.obj
    prefix = urlparse(grafana.session.url).netloc

    target = root / prefix
    target.mkdir(parents=True)

    for board in grafana.dashboards():
        filename = target / f'{board.id}.json'
        board.export(filename)



import requests


class APISession(requests.Session):
    def __init__(self, url, api_key):
        self.url = url.rstrip('/')
        self.api_key = api_key
        super().__init__()

    def request(
        self,
        method,
        url,
        params=None,
        data=None,
        headers=None,
        cookies=None,
        files=None,
        auth=None,
        timeout=None,
        allow_redirects=True,
        proxies=None,
        hooks=None,
        stream=None,
        verify=None,
        cert=None,
        json=None,
    ):
        url = self.url + url
        if headers is None:
            headers = {}
        headers['Authorization'] = f'Bearer {self.api_key}'
        return super().request(
            method,
            url,
            params,
            data,
            headers,
            cookies,
            files,
            auth,
            timeout,
            allow_redirects,
            proxies,
            hooks,
            stream,
            verify,
            cert,
            json,
        )
